@@ -16,7 +16,6 @@ from typing import Any, Iterable
 import pandas as pd
 from pandas.tseries.frequencies import to_offset as pd_to_offset
 from pandas.tseries.offsets import (
-    BusinessMixin,
     DateOffset,
     NaT,
     as_datetime,
@@ -110,6 +109,8 @@ def to_offset(rule: str) -> DateOffset:
         The rule to use to generate the offset. For options see `pandas offset aliases
         <http://pandas.pydata.org/pandas-docs/stable/user_guide/timeseries.html#timeseries-offset-aliases>`_.
 
+        Business-related offsets, such as 'B', 'C' or 'RE', are not supported.
+
     Returns
     -------
     :obj:`DateOffset`
@@ -118,11 +119,16 @@ def to_offset(rule: str) -> DateOffset:
     Raises
     ------
     ValueError
-        If unsupported offset rule is requested, e.g. all business related offsets
+        If unsupported offset rule is requested, e.g. all business-related offsets
     """
     offset = pd_to_offset(rule)
 
-    if isinstance(offset, BusinessMixin) or offset.rule_code.startswith("B"):
+    # Ignoring BusinessDay, Custom Business day and Retail offsets
+    if (
+        offset.rule_code.startswith("B")
+        or offset.rule_code.startswith("R")
+        or offset.rule_code.startswith("C")
+    ):
         raise ValueError(
             "Invalid rule for offset - Business related offsets are not supported"
         )
