@@ -15,11 +15,14 @@ from typing import Any, Callable, Dict, Iterable, List, Optional, Tuple, Union
 
 import numpy as np
 import pandas as pd
+import pint
 import xarray as xr
 from dateutil import parser
 from xarray.core.ops import inject_binary_ops
 
+from . import REQUIRED_COLS
 from .dataframe import ScmDataFrame
+from .errors import NonUniqueMetadataError
 from .filters import (
     HIERARCHY_SEPARATOR,
     datetime_match,
@@ -41,8 +44,8 @@ from .units import UnitConverter
 
 _logger = getLogger(__name__)
 
-REQUIRED_COLS = ["model", "scenario", "region", "variable", "unit"]
-"""Minimum metadata columns required by an ScmRun"""
+
+MetadataType = Dict[str, Union[str, int, float]]
 
 
 def _read_file(  # pylint: disable=missing-return-doc
@@ -290,6 +293,7 @@ class ScmRun:  # pylint: disable=too-many-public-methods
         data,
         index: Any = None,
         columns: Optional[Union[Dict[str, list], Dict[str, str]]] = None,
+        metadata: Optional[MetadataType] = None,
         **kwargs: Any,
     ):
         """
@@ -357,6 +361,14 @@ class ScmRun:  # pylint: disable=too-many-public-methods
                     ScmRun(d, columns=col_2).meta
                 )
 
+        metadata:
+            Optional dictionary of metadata for instance as a whole.
+
+            This can be used to store information such as the longer-form information
+            about a particular dataset, for example, dataset description or DOIs.
+
+            Defaults to an empty :obj:`dict` if no default metadata are provided.
+
         **kwargs:
             Additional parameters passed to :func:`_read_file` to read files
 
@@ -373,9 +385,16 @@ class ScmRun:  # pylint: disable=too-many-public-methods
         if isinstance(data, ScmRun):
             self._ts = data._ts
             self._time_points = TimePoints(data.time_points.values)
+            if metadata is None:
+                metadata = data.metadata.copy()
         else:
             self._init_timeseries(data, index, columns, **kwargs)
 
+        if self._duplicated_meta():
+            raise NonUniqueMetadataError(self.meta)
+
+        self.metadata = metadata.copy() if metadata is not None else {}
+
     def _init_timeseries(
         self,
         data,
@@ -433,6 +452,7 @@ class ScmRun:  # pylint: disable=too-many-public-methods
             :func:`copy.deepcopy` of ``self``
         """
         ret = copy.copy(self)
+        ret.metadata = copy.deepcopy(self.metadata)
         if copy_ts:
             ret._ts = [ts.copy() for ts in self._ts]
         return ret
@@ -558,6 +578,9 @@ class ScmRun:  # pylint: disable=too-many-public-methods
                     )
                 )
 
+        if self._duplicated_meta():
+            raise NonUniqueMetadataError(self.meta)
+
     def __repr__(self):
         def _indent(s):
             lines = ["\t" + line for line in s.split("\n")]
@@ -581,8 +604,20 @@ class ScmRun:  # pylint: disable=too-many-public-methods
         def func(self, other):
             if isinstance(other, ScmRun):
                 return NotImplemented
-            if not isinstance(other, numbers.Number) and len(other) != len(self):
-                raise ValueError("Incorrect length")
+
+            is_number = isinstance(other, (numbers.Number, pint.Quantity))
+            if not is_number:
+                other_ndim = len(other.shape)
+                if other_ndim == 1:
+                    if other.shape[0] != self.shape[1]:
+                        raise ValueError(
+                            "only vectors with the same number of timesteps "
+                            "as self ({}) are supported".format(self.shape[1])
+                        )
+                else:
+                    raise ValueError(
+                        "operations with {}d data are not supported".format(other_ndim)
+                    )
 
             ret = self.copy(copy_ts=False)
             ret._ts = [
@@ -592,9 +627,9 @@ class ScmRun:  # pylint: disable=too-many-public-methods
 
         return func
 
-    def drop_meta(self, columns: Union[list, str], inplace: bool = True):
+    def drop_meta(self, columns: Union[list, str], inplace: Optional[bool] = None):
         """
-        Drop metadata columns out of the Run
+        Drop meta columns out of the Run
 
         Notes
         -----
@@ -606,13 +641,25 @@ class ScmRun:  # pylint: disable=too-many-public-methods
         columns
             The column or columns to drop
         inplace
-            If True, do operation inplace and return None
+            If True, do operation inplace and return None.
+
+            The default behaviour of dropping metadata inplace will change in v0.7.0. The new behaviour in v0.7.0 will be to not
+            perform this operation in place, instead returning a copy with modified meta.
+
 
         Raises
         ------
         KeyError
             If any of the columns do not exist in the meta :class:`DataFrame`
         """
+        if inplace is None:
+            warnings.warn(
+                "drop_meta default behaviour will change to not performing operation inplace in v0.7.0. Explicitly set inplace=True to retain current behaviour",
+                DeprecationWarning,
+                2,
+            )
+            inplace = True
+
         if inplace:
             df = self
         else:
@@ -703,8 +750,11 @@ class ScmRun:  # pylint: disable=too-many-public-methods
             The value of `time_axis` would result in columns which aren't unique
         """
         df = pd.DataFrame(self.values)
-        _meta = self.meta if meta is None else self.meta[meta]
-        if check_duplicated and _meta.duplicated().any():
+
+        # Is `check_duplicated` now obsolete? I don't see how you can end up with
+        # duplicate metadata anymore. If it's not obsolete, we should add an
+        # explicit test for this behaviour (I can't see one at the moment).
+        if check_duplicated and self._duplicated_meta(meta=meta):
             raise ValueError("Duplicated meta values")
 
         if time_axis is None:
@@ -743,10 +793,17 @@ class ScmRun:  # pylint: disable=too-many-public-methods
 
         df.columns = columns
         df.columns.name = "time"
+
+        _meta = self.meta if meta is None else self.meta[meta]
         df.index = pd.MultiIndex.from_arrays(_meta.values.T, names=_meta.columns)
 
         return df
 
+    def _duplicated_meta(self, meta=None):
+        _meta = self.meta if meta is None else self.meta[meta]
+
+        return _meta.duplicated().any()
+
     def long_data(self, time_axis=None):
         """
         Return data in long form, particularly useful for plotting with seaborn
@@ -1608,7 +1665,8 @@ class ScmRun:  # pylint: disable=too-many-public-methods
         self,
         other,
         inplace: bool = False,
-        duplicate_msg: Union[str, bool] = "warn",
+        duplicate_msg: Union[str, bool] = True,
+        metadata: Optional[MetadataType] = None,
         **kwargs: Any,
     ):
         """
@@ -1626,9 +1684,15 @@ class ScmRun:  # pylint: disable=too-many-public-methods
             new :class:`ScmRun` instance with the appended data.
 
         duplicate_msg
-            If "warn", raise a warning if duplicate data is detected. If "return",
-            return the joint dataframe (including duplicate timeseries) so the user can
-            inspect further. If ``False``, take the average and do not raise a warning.
+            If ``True``, raise a ``NonUniqueMetadataError`` error so the user
+            can see the duplicate timeseries. If ``False``, take the average
+            and do not raise a warning or error. If ``"warn"``, raise a
+            warning if duplicate data is detected.
+
+        metadata
+            If not ``None``, override the metadata of the resulting :obj:`ScmRun` with ``metadata``.
+            Otherwise, the metadata for the runs are merged. In the case where there are duplicate
+            metadata keys, the values from the first run are used.
 
         **kwargs
             Keywords to pass to :func:`ScmRun.__init__` when reading
@@ -1643,7 +1707,12 @@ class ScmRun:  # pylint: disable=too-many-public-methods
         if not isinstance(other, ScmRun):
             other = self.__class__(other, **kwargs)
 
-        return run_append([self, other], inplace=inplace, duplicate_msg=duplicate_msg)
+        return run_append(
+            [self, other],
+            inplace=inplace,
+            duplicate_msg=duplicate_msg,
+            metadata=metadata,
+        )
 
     def to_iamdataframe(self) -> LongDatetimeIamDataFrame:  # pragma: no cover
         """
@@ -1767,18 +1836,31 @@ def df_append(*args, **kwargs):
     for further inspection.
 
     .. deprecated:: 0.5.0
-        :func:`df_append` will be removed in scmdata v0.6.0, it is replaced by :func:`scmdata.run.run_append`.
+        :func:`df_append` will be removed in scmdata v0.7.0, it is replaced by :func:`scmdata.run.run_append`.
     """
     warnings.warn(
-        "scmdata.run.df_append has been deprecated and will be removed in v0.6.0. Use the scmdata.run.run_append class instead",
+        "scmdata.run.df_append has been deprecated and will be removed in v0.7.0. Use the scmdata.run.run_append class instead",
         DeprecationWarning,
         2,
     )
     return run_append(*args, **kwargs)
 
 
+def _merge_metadata(metadata):
+    res = metadata[0].copy()
+
+    for m in metadata[1:]:
+        for k, v in m.items():
+            if k not in res:
+                res[k] = v
+    return res
+
+
 def run_append(
-    runs, inplace: bool = False, duplicate_msg: Union[str, bool] = "warn",
+    runs,
+    inplace: bool = False,
+    duplicate_msg: Union[str, bool] = True,
+    metadata: Optional[MetadataType] = None,
 ):
     """
     Append together many objects.
@@ -1824,9 +1906,15 @@ def run_append(
         ``None``.
 
     duplicate_msg
-        If "warn", raise a warning if duplicate data is detected. If "return", return
-        the joint :obj`ScmRun` (including duplicate timeseries) so the user can inspect
-        further. If ``False``, take the average and do not raise a warning.
+        If ``True``, raise a ``NonUniqueMetadataError`` error so the user can
+        see the duplicate timeseries. If ``False``, take the average and do
+        not raise a warning or error. If ``"warn"``, raise a warning if
+        duplicate data is detected.
+
+    metadata
+        If not ``None``, override the metadata of the resulting :obj:`ScmRun` with ``metadata``.
+        Otherwise, the metadata for the runs are merged. In the case where there are duplicate
+        metadata keys, the values from the first run are used.
 
     Returns
     -------
@@ -1863,36 +1951,32 @@ def run_append(
     for r in runs:
         if not np.array_equal(new_t, r.time_points.values):
             all_valid_times = False
+
     if not all_valid_times:
         # Time values are converted to cftime to avoid OutOfBoundsDatetime errors
         ret._time_points = TimePoints(new_t)
         new_t_cftime = ret._time_points.as_cftime()
         ret._ts = [ts.reindex(new_t_cftime) for ts in ret._ts]
 
-    if ret.meta.duplicated().any():
+    if ret._duplicated_meta():
         if duplicate_msg:
-            warn_handle_res = _handle_potential_duplicates_in_append(ret, duplicate_msg)
-            if warn_handle_res is not None:
-                return warn_handle_res  # type: ignore  # special case
+            _handle_potential_duplicates_in_append(ret, duplicate_msg)
 
         # average identical metadata
         ret._ts = ret.groupby(ret.meta_attributes).mean(axis=0)._ts
 
     ret._ts.sort(key=lambda a: a.name)
 
+    if metadata is not None:
+        ret.metadata = metadata
+    else:
+        ret.metadata = _merge_metadata([r.metadata for r in runs])
+
     if not inplace:
         return ret
 
 
 def _handle_potential_duplicates_in_append(data, duplicate_msg):
-    # If only one number contributes to each of the timeseries, we're not looking at
-    # duplicates so can return.
-    ts = data.timeseries(check_duplicated=False)
-    contributing_values = (~ts.isnull()).astype(int).groupby(ts.index.names).sum()
-    duplicates = (contributing_values > 1).any().any()
-    if not duplicates:
-        return None
-
     if duplicate_msg == "warn":
         warn_msg = (
             "Duplicate time points detected, the output will be the average of "
@@ -1901,11 +1985,8 @@ def _handle_potential_duplicates_in_append(data, duplicate_msg):
         warnings.warn(warn_msg)
         return None
 
-    if duplicate_msg == "return":
-        warnings.warn(
-            "Result contains overlapping data values with non unique metadata"
-        )
-        return data
+    if duplicate_msg and not isinstance(duplicate_msg, str):
+        raise NonUniqueMetadataError(data.meta)
 
     raise ValueError("Unrecognised value for duplicate_msg")
 
